/**
 * NamespaceFilter.java
 *
 * DOM Usage example from the O'Reilly _XML in a Nutshell_ book.
 *
 */

// we'll use the Apache Software Foundation's Xerces parser.
import org.apache.xerces.parsers.*;
import org.apache.xerces.framework.*;

// import the DOM interfaces
import org.w3c.dom.*;
import org.w3c.dom.traversal.*;
import org.xml.sax.*;

// get the necessary java support classes
import java.io.*;
import java.util.*;

public class NamespaceFilter
{
  boolean m_fDefInclude = true;
  HashMap m_hmInclude = new HashMap();

  public NamespaceFilter()
  {
  }

  public void includeNS(String strNSURI)
  {
    m_hmInclude.put(strNSURI, new Boolean(true));
  }

  public void excludeNS(String strNSURI)
  {
    m_hmInclude.put(strNSURI, new Boolean(false));
  }

  public Vector processArgs(String[] args)
  {
    Vector vecProcess = new Vector();

    for (int i = 0; i < args.length; i++) {
      if (args[i].charAt(0) == '-') {
        switch (args[i].charAt(1)) {
        case 'i': {
          if (++i < args.length) {
            includeNS(args[i]);
          }
        } break;

        case 'e': {
          if (++i < args.length) {
            excludeNS(args[i]);
          }
        } break;

        case 'x': {
          m_fDefInclude = false;
        } break;
        }
      } else {
        vecProcess.add(args[i]);
      }
    }

    return vecProcess;
  }

  public void outputNode(Node nd, PrintWriter pwOut)
  {
    switch (nd.getNodeType()) {
    case Node.ATTRIBUTE_NODE: {
      pwOut.print(" " + nd.getNodeName() + "=\"" + nd.getNodeValue() + "\"");
    } break;

    case Node.CDATA_SECTION_NODE: {
      pwOut.print("<![CDATA[" + nd.getNodeValue() + "]]>");
    } break;

    case Node.COMMENT_NODE: {
      pwOut.print("<!--" + nd.getNodeValue() + "-->");
    } break;

    case Node.DOCUMENT_TYPE_NODE: {
      DocumentType dt = (DocumentType)nd;
      pwOut.print("<!DOCTYPE " + dt.getNodeName());
      if (dt.getPublicId() == null) {
        if (dt.getSystemId() != null) {
          pwOut.print(" SYSTEM \"" + dt.getSystemId() + "\"");
        }
      } else {
        pwOut.print(" PUBLIC \"" + dt.getPublicId() + "\" \"" +
            dt.getSystemId() + "\"");
      }
      pwOut.print(">");
    } break;

    case Node.ELEMENT_NODE: {
      pwOut.print("<" + nd.getNodeName());
      NamedNodeMap nnm = nd.getAttributes();
      for (int i = 0; i < nnm.getLength(); i++) {
        outputNode(nnm.item(i), pwOut);
      }
      pwOut.print('>');

      NodeList nl = nd.getChildNodes();
      for (int i = 0; i < nl.getLength(); i++) {
        outputNode(nl.item(i), pwOut);
      }
      pwOut.print("</" + nd.getNodeName() + ">");
    } break;

    case Node.ENTITY_NODE:
    case Node.NOTATION_NODE:
    break;

    case Node.ENTITY_REFERENCE_NODE: {
      pwOut.print("&" + nd.getNodeName() + ";");
    } break;

    case Node.PROCESSING_INSTRUCTION_NODE: {
      ProcessingInstruction pi = (ProcessingInstruction)nd;
      pwOut.print("<?"  + pi.getTarget());
      if (pi.getData() != null) {
        pwOut.print(" " + pi.getData());
      }

      pwOut.print("?>");
    } break;

    case Node.TEXT_NODE: {
      pwOut.print(nd.getNodeValue());
    } break;
    }
  }

  public void processDoc(String strURI, PrintWriter pwOut) throws SAXException,
    IOException
  {
    // create a new parser
    DOMParser dp = new DOMParser();

    // parse the document and get the DOM Document interface
    dp.parse(strURI);
    Document doc = dp.getDocument();
    pruneNode(doc.getDocumentElement());
    outputNode(doc.getDocumentElement(), pwOut);
  }

  void pruneNode(Node nd)
  {
    switch (nd.getNodeType()) {
    case Node.ELEMENT_NODE: {
      boolean fKeep = m_fDefInclude;

      String strName = nd.getNodeName();
      if (m_hmInclude.containsKey(nd.getNamespaceURI())) {
        fKeep = ((Boolean)m_hmInclude.get(nd.getNamespaceURI())).booleanValue();
      }

      NamedNodeMap nnm = nd.getAttributes();

      int i = 0;
      while (i < nnm.getLength()) {
        Node ndAttr = nnm.item(i);

        if (ndAttr.getPrefix() != null && ndAttr.getPrefix().indexOf("xml") != 0) {
          String strURICheck = ndAttr.getNamespaceURI();
          Boolean fKeepAttr = ((Boolean)m_hmInclude.get(strURICheck));

          if (fKeepAttr == null ? m_fDefInclude : fKeepAttr.booleanValue()) {
            i++;
          } else {
            nd.getAttributes().removeNamedItemNS(ndAttr.getNamespaceURI(),
                ndAttr.getLocalName());
          }
        } else {
          i++;
        }
      }

      NodeList nl = nd.getChildNodes();

      for (i = 0; i < nl.getLength(); i++) {
        pruneNode(nl.item(i));
      }

      if (!fKeep) {
        collapseBranch(nd);
      }
    } break;
    }
  }

  void collapseBranch(Node nd)
  {
    Document doc = nd.getOwnerDocument();

    if (nd.equals(doc.getDocumentElement())) {
      // can't really collapse the document element
      return;
    }

    DocumentFragment df = doc.createDocumentFragment();

    NodeList nl = nd.getChildNodes();

    int i = 0;
    while (i < nl.getLength()) {
      Node ndChild = nl.item(i);

      if (ndChild.getNodeType() == Node.ELEMENT_NODE) {
        df.appendChild(ndChild.getParentNode().removeChild(ndChild));
      } else {
        i++;
      }
    }

    Node ndParent = nd.getParentNode();
    nl = df.getChildNodes();

    while (nl.getLength() > 0) {
      ndParent.insertBefore(df.removeChild(nl.item(0)), nd);
    }

    ndParent.removeChild(nd);
  }

  public static void main(String[] args) {
    NamespaceFilter nf = new NamespaceFilter();
    Vector vecProcess = nf.processArgs(args);
    PrintWriter pwOut = new PrintWriter(new OutputStreamWriter(System.out));

    try {
      Iterator i = vecProcess.iterator();

      while (i.hasNext()) {
        nf.processDoc((String)i.next(), pwOut);
      }

      pwOut.close();
    } catch (SAXParseException spe) {
      System.err.println("parse error: " + spe.getSystemId() + ": line " +
          spe.getLineNumber() + ": col: " + spe.getColumnNumber() + ": " +
          spe.getMessage());
    } catch (SAXException se) {
      System.err.println(se);
    } catch (IOException ioe) {
      System.err.println(ioe);
    }
  }
}